#ifndef __GSHTTP_H__
#define __GSHTTP_H__

#include <glib.h>
#include <gnet.h>

typedef struct _GSHTTP GSHTTP;
typedef struct _GSHTTP_event GSHTTP_event;
typedef struct _GSHTTP_client GSHTTP_client;

typedef gboolean (*GSHTTPFunc)(GSHTTP *gsh, GSHTTP_event *evt, gpointer user_data);

struct http_req {
	gchar			*method;	/* command */
	GURI			*uri;		/* URI */
	GHashTable		*hdr;		/* headers */

	int			major;		/* version major */
	int			minor;		/* version minor */
};

enum GSHTTP_event_type {
	http_evt_req,				/* HTTP request */
	http_evt_exit				/* TCP server exit */
};

struct _GSHTTP_event {
	enum GSHTTP_event_type	type;
	union {
		GSHTTP_client	*cli;
	} u;
};

/* internal client socket state */
enum client_state {
	evt_get_req,				/* get request line */
	evt_get_hdr,				/* get header line */
	evt_dispose,				/* dispose of client */
};

struct _GSHTTP_client {
	struct _GSHTTP		*gsh;		/* server handle */

	enum client_state	state;		/* see evt_xxx above */
	GConn			*conn;		/* socket */

	struct http_req		*req;		/* parsed HTTP request */

	GString			*tmp_hdr;	/* temporary header
						 * accumulation buffer */
};

struct _GSHTTP {
	GServer			*gsrv;		/* TCP server */
	unsigned int		ref;		/* obj ref count */
	int			idle_timeout;	/* read idle timeout */

	GSHTTPFunc		func;		/* user callback */
	gpointer		user_data;	/* user data */
};

extern GSHTTP *gshttp_new(const GInetAddr *iface, gint port, GSHTTPFunc func,
		   gpointer user_data);
extern void gshttp_delete(GSHTTP *gsh);
extern void gshttp_unref(GSHTTP *gsh);
extern void gshttp_ref(GSHTTP *gsh);

#endif /* __GSHTTP_H__ */
