
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <libgen.h>
#include <glib.h>
#include <gnet.h>
#include "http.h"

struct server;

#define PROGRAM_NAME "httpd"

struct server {
	int			port;
	int			idle_timeout;
};

static gchar *file_path;
static gchar *file_wire_data;
static gsize file_wire_len;

static char *file_not_found =
"HTTP/1.1 404 not found\r\n"
"Connection: close\r\n"
"Content-type: text/html\r\n"
"\r\n"
"<html><head><title>404 file not found</title></head>\r\n"
"<body><h1>File not found</h1>\r\n"
"<h2>The requested resource was not found.</h2>\r\n"
"</body></html>\r\n";

static const char *file_header_fmt =
"HTTP/1.1 200 OK\r\n"
"Connection: close\r\n"
"Content-type: %s\r\n"
"Content-length: %lu\r\n"
"\r\n";

static gboolean http_serve_file(GSHTTP_client *cli, struct server *srv)
{
	gnet_conn_write(cli->conn, file_wire_data, file_wire_len);

	return FALSE;
}

static gboolean http_request(GSHTTP_client *cli, struct server *srv)
{
	struct http_req *req = cli->req;

	if (!strcmp(file_path, req->uri->path))
		return http_serve_file(cli, srv);

	gnet_conn_write(cli->conn, file_not_found, strlen(file_not_found));
	return FALSE;
}

static gboolean http_event(GSHTTP *gsh, GSHTTP_event *evt, gpointer user_data)
{
	struct server *srv = user_data;
	gboolean more_req = FALSE;

	switch (evt->type) {
	case http_evt_req:
		more_req = http_request(evt->u.cli, srv);
		break;
	default:
		/* do nothing */
		break;
	}

	return more_req;
}

static void build_resp_data(const char *argv_fn, const char *file_type)
{
	gchar *resp_hdr, *resp;
	gsize len, hdr_len;
	gchar *file_data;
	gsize file_len;

	if (!g_file_get_contents(argv_fn, &file_data, &file_len, NULL))
		exit(1);

	resp_hdr = g_strdup_printf(file_header_fmt, file_type,
				   (unsigned long) file_len);

	hdr_len = strlen(resp_hdr);
	len = hdr_len + file_len;

	resp = g_malloc(len);
	memcpy(resp, resp_hdr, hdr_len);
	memcpy(resp + hdr_len, file_data, file_len);

	file_wire_data = resp;
	file_wire_len = len;

	g_free(resp_hdr);
	g_free(file_data);
}

static struct server global_srv = {
	.port			= 8080,
	.idle_timeout		= 15 * 60 * 1000,
};

int main (int argc, char *argv[])
{
	struct server *srv = &global_srv;
	GMainLoop *loop = g_main_loop_new(NULL, FALSE);
	char *basec, *bname;

	if (argc != 3) {
		fprintf(stderr, "usage: %s FILE CONTENT-TYPE\n", argv[0]);
		return 1;
	}

	basec = strdup(argv[1]);
	bname = basename(basec);

	file_path = g_strdup_printf("/%s", bname);
	if (!file_path)
		return 1;

	build_resp_data(argv[1], argv[2]);

	gnet_init();

	loop = g_main_loop_new(NULL, FALSE);

	gshttp_new(NULL, srv->port, http_event, srv);

	g_main_loop_run(loop);

	return 0;
}

